$lines
     remark  program COMPOUND.INTEREST(input = CRT, output = CRT)

	comment
	   This is a program used to solve compound interest problems.
	   The program was written in Structured Basic by Jeff Brown.
	   Version nos. for the software are:

	      CP/M  2.2
	      S-Basic v5.4b, document revision 4.0 for KayPro II
	      Perfect Writer 1.20

	   Given any three of the four variables (Initial Amount, Final
	   Amount, Interest Rate, Numbers) it will determine the missing
	   one's value.  The basic formulas used to calculate these
	   variables are:

	      Discrete Compounding        V = A * (1.0 + r/n)**mn

	      Continuous Compounding      V = A * e**(rm)

	   where V is the final amount, A is the initial amount, r is the
	   interest rate (a decimal fraction), n is the number of
	   compounding periods per year, and m is the number of years.  Note
	   that e stands for the value 2.71828...
	end of comment


	$constant  CRT = 0		;logical device, 0 = con:
	$constant  SEND.BEEP = 07	;ascii code for bell
	$constant  FALSE = 0		;logical statement false
	$constant  TRUE = 0FFFFH	;logical statement true
	$constant  DELAY = 2000		;loop count for screen refresh holdoff

	var  CALC.SELECTION		;calculation type selected
	     PERIOD.CHAR		;compounding period character selected
	        = char
	var  PERIOD			;compounding period value selected
	        = real.double
	var  MONEY.PRINT.FORMAT ;printing format of monetary output
	     RATE.PRINT.FORMAT	;printing format of percentage output
	     YEARS.PRINT.FORMAT	;printing format of number of years output
	        = string:20

	MONEY.PRINT.FORMAT = "$$##,###,###,###.##"
	RATE.PRINT.FORMAT = "####.##%"
	YEARS.PRINT.FORMAT = "####.## years\."


$list
	$include PRIMLIB HOME.AND.CLEAR.SCREEN
	$include PRIMLIB CHECK.VALID.CHAR
	$include PRIMLIB CHECK.VALID.REAL.NUMBER
	$include PRIMLIB CONVERT.LOWER.TO.UPPER
	$include PRIMLIB INVALID.INPUT
	$include PRIMLIB RETURN.TO.MAIN.MENU
$list


	procedure INTRO.TO.COMPOUND.INTEREST
	   comment
	      This module prints an introduction to the COMPOUND INTEREST
	      Program.
	   end of comment

	   HOME.AND.CLEAR.SCREEN
	   print #CRT; "Welcome to the Compound Interest Solver"
	   print #CRT; "How can I help you?"
	   print #CRT

	end of INTRO.TO.COMPOUND.INTEREST


	function GET.CALCULATION.SELECTION = char
	   comment
	      This module prints the menu selection available to the user
	      and then gets his response.
	   end of comment

	   var  THE.SELECTION	;user's character response to menu
	           = char
	   var  I		;loop index
	           = integer

	   text CRT,&
	If you can provide me with some information, I can
	calculate the following for you:

	   <F>inal amount - knowing initial amount, interest
	              rate, and number of years.
	   <I>initial amount - knowing final amount, interest
	              rate, and number of years.
	   <R>ate - knowing initial amount, final amount, and
	              number of years.
	   <Y>ears - knowing initial amount, final amount, and
	              interest rate.

	Just type your selection below or type <E> to exit.

	&
	   remark  turn off auto echo of user input
	   echo off

	   input1 #CRT; "Selection"; THE.SELECTION

	   remark  turn on auto echo of user input
	   echo on

	   THE.SELECTION = CONVERT.LOWER.TO.UPPER(THE.SELECTION)
	   if not CHECK.VALID.CHAR("FIREYFIREY",THE.SELECTION)  then  THE.SELECTION = 'N'
	   case THE.SELECTION of
	      'N':  begin
	               remark  invalid entry
	               HOME.AND.CLEAR.SCREEN
	               INVALID.INPUT 'M'
	            end of case 'N'
	      'F':  print #CRT; "Final Amount"
	      'I':  print #CRT; "Initial Amount"
	      'R':  print #CRT; "Rate"
	      'Y':  print #CRT; "Years"
	      'E':  print #CRT; "Exit"
	   end of case THE.SELECTION
	   if (THE.SELECTION <> 'N')  then
	      begin
	         for I = 1 to DELAY	remark  wait loop
	         next I
	         HOME.AND.CLEAR.SCREEN
	      end of if-then

	end = THE.SELECTION

	procedure PRINT.PERIOD(THE.SELECTION = char)
	   comment
	      This primitive translates the period character into the
	      full word and prints it.
	   end of comment

	   case THE.SELECTION of
	      'N':  remark  error, do nothing
	      'C':  print #CRT; "Continuous"
	      'D':  print #CRT; "Daily"
	      'W':  print #CRT; "Weekly"
	      'M':  print #CRT; "Monthly"
	      'Q':  print #CRT; "Quarterly"
	      'S':  print #CRT; "Semiannually"
	      'Y':  print #CRT; "Yearly"
	      'E':  print #CRT; "Exit"
	   end of case THE.SELECTION

	end of PRINT.PERIOD


	function GET.PERIOD.SELECTION = char
	   comment
	      This module gets the user's compounding period selection.
	   end of comment

	   var  THE.SELECTION	;user's character response to menu
	           = char
	   var  I		;loop index
	           = integer

	   text CRT,&
	Which type of compounding period would you like?

	   <C>ontinuously
	   <D>aily
	   <W>eekly
	   <M>onthly
	   <Q>uarterly
	   <S>emiannually
	   <Y>early

	Type your selection below or type <E> to exit from this menu.
	&
	   remark  turn off auto echo of user input
	   echo off

	   input1 #CRT; "Selection"; THE.SELECTION

	   remark  turn on auto echo of user input
	   echo on

	   THE.SELECTION = CONVERT.LOWER.TO.UPPER(THE.SELECTION)
	   if not CHECK.VALID.CHAR("CDWMQSYECD",THE.SELECTION)  then
	         begin
	            remark  indicate error
	            THE.SELECTION = 'N'
	            HOME.AND.CLEAR.SCREEN
	            INVALID.INPUT 'M'
	         end
	      else
	         begin
	            PRINT.PERIOD THE.SELECTION
	            for I = 1 to DELAY	remark  wait loop
	            next I
	            HOME.AND.CLEAR.SCREEN
	         end of if-then-else
	
	end = THE.SELECTION


	function PERIOD.VALUE = real.double
	   comment
	      This primitive translates the period character into a numeric
	      value
	   end of comment

	   var  THE.VALUE		;period value
	           = real.double

	   case PERIOD.CHAR of
	      'N':  INVALID.INPUT 'G'
	      'C':  begin
	               remark  set to negative number to signal continuous
	               THE.VALUE = -1.0
	            end of case 'C'
	      'D':  begin
	               remark  Compounding period is daily
	               THE.VALUE = 365.0
	            end of case 'D'
	      'W':  begin
	               remark  Compounding period is weekly
	               THE.VALUE = 52.0
	            end of case 'W'
	      'M':  begin
	               remark  Compounding period is monthly
	               THE.VALUE = 12.0
	            end of case 'M'
	      'Q':  begin
	               remark  Compounding period is quarterly
	               THE.VALUE = 4.0
	            end of case 'Q'
	      'S':  begin
	               remark  Compounding period is semiannually
	               THE.VALUE = 2.0
	            end of case 'S'
	      'Y':  begin
	               remark  Compounding period is annually
	               THE.VALUE = 1.0
	            end of case 'Y'
	      'E':  INVALID.INPUT 'G'
	   end of case PERIOD.CHAR

	end = THE.VALUE


	function GET.REAL(PROMPT=char;LOWER.BOUND,UPPER.BOUND=real.double)=real.double
	   comment
	      This primitive gets a real number from the user and checks
	      is against a boundary set.
	   end of comment

	   var  THE.NUMBER	;number gotten from user
	           = real.double
	   var  INPUT.IS.OK	;logical variable holding input check result
	           = integer

	   if not(CHECK.VALID.CHAR("FIRYFIRYFI",PROMPT))  then
	      INVALID.INPUT 'G'

	   repeat
	      begin
	         case PROMPT of
	            'F':  input1 #CRT; "        What is the Final Amount"; THE.NUMBER
	            'I':  input1 #CRT; "        What is the Initial Amount"; THE.NUMBER
	            'R':  input1 #CRT; "        What is the Interest Rate (i.e. 5, 8.5 ...)"; THE.NUMBER
	            'Y':  input1 #CRT; "        How many years will it accummulate"; THE.NUMBER
	         end of case PROMPT

	         INPUT.IS.OK = CHECK.VALID.REAL.NUMBER(LOWER.BOUND,UPPER.BOUND,THE.NUMBER)
	         if not INPUT.IS.OK  then  INVALID.INPUT 'R'
	      end
	   until  INPUT.IS.OK

	   print #CRT

	end = THE.NUMBER
   

	procedure FIND.FINAL.AMOUNT
	   comment
	      This module calculates the final amount, given the compounding
	      period, by inquiring as to the initial amount, the interest
	      rate, and the number of years.
	   end of comment

	   var  FINAL.AMOUNT		;
	        INITIAL.AMOUNT		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	           = real.double

	   print #CRT; "Compound Interest Solver invoking FINAL AMOUNT formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   INITIAL.AMOUNT = GET.REAL('I',0.0,100000000.00)
	   INTEREST.RATE = GET.REAL('R',0.0,1000.00) / 100.0
	   NUMBER.OF.YEARS = GET.REAL('Y',0.0,1000.00)

	   remark  The Final Amount is computed using one formula for
	   remark  discrete compounding (Period char <> C) and another for
	   remark  continuous compounding

	   if ( PERIOD.CHAR <> 'C' )  then  FINAL.AMOUNT =INITIAL.AMOUNT * (1.0 + INTEREST.RATE / PERIOD) ** (NUMBER.OF.YEARS * PERIOD)
	      else  FINAL.AMOUNT = INITIAL.AMOUNT * exp(INTEREST.RATE * NUMBER.OF.YEARS)

	   print #CRT
	   print #CRT; "        The FINAL AMOUNT is ";
	   print using MONEY.PRINT.FORMAT; #CRT; FINAL.AMOUNT
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.FINAL.AMOUNT


	procedure FIND.INITIAL.AMOUNT
	   comment
	      This module calculates the initial amount, given the compounding
	      period, by inquiring as to the final amount, the interest
	      rate, and the number of years.
	   end of comment

	   var  FINAL.AMOUNT		;
	        INITIAL.AMOUNT		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	           = real.double

	   print #CRT; "Compound Interest Solver invoking INITIAL AMOUNT formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   FINAL.AMOUNT = GET.REAL('F',0.0,100000000.00)
	   INTEREST.RATE = GET.REAL('R',0.0,1000.00) / 100.0
	   NUMBER.OF.YEARS = GET.REAL('Y',0.0,1000.00)

	   remark  The Initial Amount is computed using one formula for
	   remark  discrete compounding (Period char <> C) and another for
	   remark  continuous compounding

	   if ( PERIOD.CHAR <> 'C' )  then  INITIAL.AMOUNT = FINAL.AMOUNT / (1.0 + INTEREST.RATE / PERIOD) ** (NUMBER.OF.YEARS * PERIOD)
	      else  INITIAL.AMOUNT = FINAL.AMOUNT / exp(INTEREST.RATE * NUMBER.OF.YEARS)

	   print #CRT
	   print #CRT; "        The INITIAL AMOUNT is ";
	   print using MONEY.PRINT.FORMAT; #CRT; INITIAL.AMOUNT
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.INITIAL.AMOUNT


	procedure FIND.INTEREST.RATE
	   comment
	      This module calculates the interest rate, given the compounding
	      period, by inquiring as to the initial amount, the final amount,
	      and the number of years.
	   end of comment

	   var  FINAL.AMOUNT		;
	        INITIAL.AMOUNT		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	           = real.double

	   print #CRT; "Compound Interest Solver invoking INTEREST RATE formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   INITIAL.AMOUNT = GET.REAL('I',1.0,100000000.00)
	   FINAL.AMOUNT = GET.REAL('F',INITIAL.AMOUNT,100000000.00)
	   NUMBER.OF.YEARS = GET.REAL('Y',0.01,1000.00)

	   remark  The Interest Rate is computed using one formula for
	   remark  discrete compounding (Period char <> C) and another for
	   remark  continuous compounding

	   if ( PERIOD.CHAR <> 'C' )  then  INTEREST.RATE = (((FINAL.AMOUNT/INITIAL.AMOUNT)**(1.0/(NUMBER.OF.YEARS*PERIOD)) - 1.0) * PERIOD) * 100.0
	      else  INTEREST.RATE = (log(FINAL.AMOUNT/INITIAL.AMOUNT) / NUMBER.OF.YEARS) * 100.0

	   print #CRT
	   print #CRT; "        The INTEREST RATE is ";
	   print using RATE.PRINT.FORMAT; #CRT; INTEREST.RATE
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.INTEREST.RATE


	procedure FIND.NUMBER.OF.YEARS
	   comment
	      This module calculates the number of years, given the
	      compounding period, by inquiring as to the initial amount,
	      the final amount, and the interest rate.
	   end of comment

	   var  FINAL.AMOUNT		;
	        INITIAL.AMOUNT		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	           = real.double

	   print #CRT; "Compound Interest Solver invoking NUMBER OF YEARS formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   INITIAL.AMOUNT = GET.REAL('I',1.0,100000000.00)
	   FINAL.AMOUNT = GET.REAL('F',INITIAL.AMOUNT,100000000.00)
	   INTEREST.RATE = GET.REAL('R',0.001,1000.00) / 100.0

	   remark  The Number of Years is computed using one formula for
	   remark  discrete compounding (Period char <> C) and another for
	   remark  continuous compounding

	   if ( PERIOD.CHAR <> 'C' )  then  NUMBER.OF.YEARS = ( log(FINAL.AMOUNT/INITIAL.AMOUNT) / log(1.0 + INTEREST.RATE/PERIOD) ) / PERIOD
	      else  NUMBER.OF.YEARS = log(FINAL.AMOUNT/INITIAL.AMOUNT) / INTEREST.RATE

	   print #CRT
	   print #CRT; "        The NUMBER OF YEARS is ";
	   print using YEARS.PRINT.FORMAT; #CRT; NUMBER.OF.YEARS
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.NUMBER.OF.YEARS

	procedure TERMINATE.COMPOUND.INTEREST
	   comment
	      This module closes off and exits from the COMPOUND INTEREST
	      Program.
	   end of comment

	   print #CRT; "Compound Interest Solver saying goodbye!"

	end of TERMINATE.COMPOUND.INTEREST


	begin
	   INTRO.TO.COMPOUND.INTEREST
	   repeat
	      begin
	         CALC.SELECTION = GET.CALCULATION.SELECTION
	         if ((CALC.SELECTION <> 'N') and (CALC.SELECTION <> 'E')) then
	            begin
	               repeat
	                  PERIOD.CHAR = GET.PERIOD.SELECTION
	               until PERIOD.CHAR <> 'N'
	               if ( PERIOD.CHAR = 'E' )  then  CALC.SELECTION = 'N'
	                  else  PERIOD = PERIOD.VALUE
	            end
	         case CALC.SELECTION of
	            'N':  remark  do nothing
	            'F':  FIND.FINAL.AMOUNT
	            'I':  FIND.INITIAL.AMOUNT
	            'R':  FIND.INTEREST.RATE
	            'Y':  FIND.NUMBER.OF.YEARS
	            'E':  remark  termination
	         end of case CALC.SELECTION
	      end 
	   until CALC.SELECTION = 'E'
	   TERMINATE.COMPOUND.INTEREST
	   stop
	end
TEREST
	      Pro